public class MicroFabControl {
	public final static double MIN_SECS_BETWEEN_STEPS = 0.00001;
	public final static double STEPS_PER_REV = 100000;
	public final static double MM_PER_REV = 2.0;
    // How long it takes to read the ADC
    public final static double SECONDS_PER_ADC = .0003;
	// digital output bits
	public final static int DOUT_Z_STEP = 0x01;
	public final static int DOUT_Z_CCW = 0x02;
	public final static int DOUT_X_STEP = 0x04;
	public final static int DOUT_X_CCW = 0x08;
	public final static int DOUT_Y_STEP = 0x10;
	public final static int DOUT_Y_CCW = 0x20;
	public final static int DOUT_STANDBY = 0x40;
	public final static int DOUT_VOLTAGE_TRIGGER = 0x80;
	// voltage trigger loop
	public final static int VTRIGGER_STATES = 3;
	int _vTriggerState = 0;
	public int vTriggerState() {return _vTriggerState;}
	// ADC input channels
    public final static int ADC_CURRENT = 1;	

    public RealTime io;

	// These are updated at each motor step and tracks the positions
	int _xSteps = 0;
	int _ySteps = 0;
	int _zSteps = 0;
	public int xSteps() {return _xSteps;}
	public int ySteps() {return _ySteps;}
	public int zSteps() {return _zSteps;}
	
	int _doutState;
	public int doutState() {return _doutState;}
	
	// This initialzes with DOUT_STANDBY on
	MicroFabControl ()
	{
		io = new RealTime();
		io.DigPrtConfig();
		
		doutSetBits (DOUT_STANDBY);

		// The DigPrtConfig did one V trigger once, so reset.  We really should set this
		//   based on voltage feedback.
		setVTriggerState (VTRIGGER_STATES - 1);
		_vTriggerState = 0;
		
	}

	// Set _doutState with the bits in bitMask and send to digital output
	public void doutSetBits (int bitMask) {
	    _doutState |= bitMask;
	    io.DigOut (_doutState);
	}
	
	// Clear _doutState of the bits in bitMask and send to digital output
	public void doutClearBits (int bitMask) {
	    _doutState |= bitMask;
	    _doutState -= bitMask;
	    io.DigOut (_doutState);
	}

    // Trigger the voltage and update _vTriggerState, looping at VTRIGGER_STATES
    public void nextVTrigger ()
    {
		RealTime.timer(MIN_SECS_BETWEEN_STEPS);
		
        doutSetBits (DOUT_VOLTAGE_TRIGGER);
        // Pause for timer interval
		RealTime.timer(-1);
        doutClearBits (DOUT_VOLTAGE_TRIGGER);
        
        _vTriggerState++;
        if (_vTriggerState >= VTRIGGER_STATES)
          _vTriggerState = 0;
    }
    
    // Calls nextVTrigger until _vTriggerState equals state.
    public void setVTriggerState (int state)
    {
        if (state < 0 || state >= VTRIGGER_STATES)
          // Invalide value, so do nothing
          return;
          
        while (_vTriggerState != state)
          nextVTrigger();
    }
    
	// Move X one step clockwise or counterclockwise based on ccw, updating _xSteps
	public void xStep (boolean ccw)
	{
		if (ccw) {
		  doutSetBits (DOUT_X_CCW);
		  _xSteps--;
		}
		else {
		  doutClearBits (DOUT_X_CCW);
		  _xSteps++;
		}
		
		// pulse.  The time of calling doutSetBits is enough for the pulse width
        doutSetBits (DOUT_X_STEP);
        doutClearBits (DOUT_X_STEP);
	}
	
	// Move Y one step clockwise or counterclockwise based on ccw, updating _ySteps
	public void yStep (boolean ccw)
	{
		if (ccw) {
		  doutSetBits (DOUT_Y_CCW);
		  _ySteps--;
		}
		else {
		  doutClearBits (DOUT_Y_CCW);
		  _ySteps++;
		}
		
		// pulse.  The time of calling doutSetBits is enough for the pulse width
        doutSetBits (DOUT_Y_STEP);
        doutClearBits (DOUT_Y_STEP);
	}
	
	// Move Z one step clockwise or counterclockwise based on ccw, updating _zSteps
	public void zStep (boolean ccw)
	{
		if (ccw) {
		  doutSetBits (DOUT_Z_CCW);
		  _zSteps--;
		}
		else {
		  doutClearBits (DOUT_Z_CCW);
		  _zSteps++;
		}

		// pulse.  The time of calling doutSetBits is enough for the pulse width
        doutSetBits (DOUT_Z_STEP);
        doutClearBits (DOUT_Z_STEP);
	}
	
	// Use RealTime.second to wait the given amount of seconds
	public static void waitSeconds (double seconds)
	{
	    double end = RealTime.second() + seconds;
	    
	    while (RealTime.second() < end);
	}

    public void zeroizeX ()
    {
        _xSteps = 0;
    }

    public void zeroizeY ()
    {
        _ySteps = 0;
    }

    public void zeroizeZ ()
    {
        _zSteps = 0;
    }
}

